#!/bin/sh

# Terminal width: 132 columns.
# Function definitions end at comment "# Program proper start."

#
# Print usage and exit.
#
usage() {
	usageflag="$1"
	usageself="$( basename "$0" )"
	usagemessage="$( cat <<- HERE
	Apply an emergency fix allowing Make to work when files.mk is either corrupted
	or missing.

	Usage:

	${usageself} -h|-?
	${usageself} -c

	-h or -?     Show this help text and exit without error.
	-c           Re-create the "files.mk.if-make-aborts" file.
	HERE
	)"
	if [ -n "${usageflag}" ]
	then printf "%s\n" "${usagemessage}"; exit 0
	else printf "%s\n" "${usagemessage}" >&2; exit 1
	fi
}


#
# Print an error message and exit.
#
errmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" | fold -s >&2
	exit 1
}


#
# Process command line options and initialize variables.
#
setoptions() {
	while getopts 'ch?' c
	do
		case $c in
		c)  makefixfile   ;;
		h)  usage help    ;;
		?)  usage help    ;;
		esac
	done
}


#
# Re-create files.mk.if-make-aborts.
#
makefixfile() {
	if [ -d "files.mk.if-make-aborts" ]; then errmsg "File 'files.mk.if-make-aborts' is a directory."; fi

	printf "%s" "Writing \`make\` fix file 'files.mk.if-make-aborts' ..."
	if ! cat /dev/null > "files.mk.if-make-aborts"; then printf "%s\n" "failed."; exit 1; fi

	# Can't simply `cat <<- HERE` these lines because *all* leading tabs get
	# eaten, leading `make` to quit with "Need an operator" errors.
	#
	l1="#"
	l2="# Make targets for each source file."
	l3="#"
	l4="# DO NOT EDIT THIS FILE!"
	l5="# This file is overwritten by the \"files\" make target."
	l6="#"
	l7=""
	l8="#"
	l9="# If make aborts because files.mk is deleted or corrupted somehow, then"
	l10="# copy and save this file as \"files.mk\" then run the following command:"
	l11="#"
	l12="#     make files"
	l13="#"
	l14="# This will give make a complete Makefile allowing the \"files\" target"
	l15="# to rebuild a valid \"files.mk\" file."
	l16="#"
	l17=""
	l18="pages:"
	l19="	@echo \"Create \"\${SRC_DIR}/index.txt\" and re-run \"make files\" first."
	l20="	@false"
	l21=""

	# Shut up, shellcheck.  I know these appear unused (SC2034) or referenced but not assigned (SC2154).
	l="$l1"; l="$l2"; l="$l3"; l="$l4"; l="$l5"; l="$l6"; l="$l7"; l="$l8"; l="$l9"; l="$l10"; l="$l11"; l="$l12"; l="$l13";
	l="$l14"; l="$l15"; l="$l16"; l="$l17"; l="$l18"; l="$l19"; l="$l20"; l="$l21"; line="$l"; printf "%s" "$line" >/dev/null

	i=1; while [ "21" -ge "${i}" ]
	do
		# Eval-magic "line=$l1" through "line=$l21" like iterating through a pseudo-array.
		#
		eval line="\$l${i}"
		if ! printf "%s\n" "${line}" >> "files.mk.if-make-aborts"; then printf "%s\n" "failed."; exit 1; fi
		i=$(( i + 1 ))
	done

	printf "%s\n" "done."
	exit 0
}


#
# Program proper start.
#
# Process command line options.
#
setoptions "$@"


#
# Check if the fix can be applied.
#
file="files.mk"
efix="files.mk.if-make-aborts"

if [ ! -e "${efix}" ]; then errmsg "File '${efix}' not found.  Try running '$0 -c' ?"; fi
if [ ! -r "${efix}" ]; then errmsg "File '${efix}' not readable.  Check permissions and try running '$0 -c' ?"; fi
if [ ! -s "${efix}" ]; then errmsg "File '${efix}' is empty.  Try running '$0 -c' ?"; fi
if [   -d "${file}" ]; then errmsg "File '${file}' is a directory."; fi
if [   -d "${efix}" ]; then errmsg "File '${efix}' is a directory."; fi

if [ -e "${file}" ]; then difftest="$( diff "${file}" "${efix}" 2>/dev/null )"; else difftest="${file} does not exist."; fi
if [ -z "${difftest}" ]; then errmsg "Files '${file}' and '${efix}' are the same."; fi

#
# Apply the fix.
#
if [ -e "${file}" ]
then
	printf "%s\n" "About to delete file '${file}' and replace with emergency \`make\` fix file '${efix}'." | fold -s
	printf "%s "  "Proceed? (y/N)"
	read -r yn
	yn=${yn#${yn%%[ynYN]*}}; yn=${yn%%[!ynYN]*}; yn=${yn%${yn#?}}
else
	yn="y"
fi
if [ "y" != "${yn}" ] && [ "Y" != "${yn}" ]; then printf "%s\n" "Not done."; exit 0; fi

if cp -f -- "${efix}" "${file}"
then printf "%s\n" "Done.  Run \`make files\` to rebuild the files target."
else printf "%s\n" "Failed."; exit 1
fi

# End.

