#!/bin/sh

# Terminal width: 132 columns.
# Function definitions end at comment "# Program proper start."

#
# Print usage and exit.
#
usage() {
	usageflag="$1"
	usageself="$( basename "$0" )"
	usagemessage="$( cat <<- HERE
	Rebuild files.mk, which contains the "pages" make target and the individual
	file make targets.

	Usage:

	${usageself} -h|-?
	${usageself} -s src_dir -d dest_dir -t temp_dir -p tpl_dir

	-h or -?     Show this help text and exit without error.
	-s src_dir   Directory src_dir containing files to convert to HTML pages.
	-d dest_dir  Directory dest_dir containing the converted and staged HTML pages.
	-t temp_dir  Directory temp_dir to use for temporary scratch files.
	-p tpl_dir   Directory tpl_dir containing HTML page template M4 files.

	Directory options are either absolute paths or relative to the current
	directory, not the directory in which ${usageself} lives.
	HERE
	)"
	if [ -n "${usageflag}" ]
	then printf "%s\n" "${usagemessage}"; exit 0
	else printf "%s\n" "${usagemessage}" >&2; exit 1
	fi
}


#
# Print an error message, then print usage and exit.
#
errmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" >&2
	usage
}


#
# Remove a file or show an error message.
#
rmfile() {
	if [ -z "$1" ]; then return; fi
	if [ -e "$1" ]; then rm -f -- "$1" || { errmsg "Can't rm file '$1'."; }; else errmsg "File to rm '$1' doesn't exist."; fi
}


#
# Remove an empty directory or show an error message.
#
rmdirectory() {
	if [ -z "$1" ]; then return; fi

	if [ -d "$1" ]
	then rmdir -- "$1" || { errmsg "Can't rmdir directory '$1'."; }
	else errmsg "Directory to rmdir '$1' doesn't exist."
	fi
}


#
# Process command line options and initialize variables.
#
setoptions() {
	unset SRC_DIR DST_DIR TMP_DIR TPL_DIR FILE

	while getopts 's:d:t:p:h?' c
	do
		case $c in
		s)  SRC_DIR="${OPTARG}"  ;;
		d)  DST_DIR="${OPTARG}"  ;;
		t)  TMP_DIR="${OPTARG}"  ;;
		p)  TPL_DIR="${OPTARG}"  ;;
		h)  usage help           ;;
		?)  usage help           ;;
		esac
	done

	[ -z "${SRC_DIR}" ] && { errmsg "Missing -s option: no source directory."; }
	[ -z "${DST_DIR}" ] && { errmsg "Missing -d option: no destination directory."; }
	[ -z "${TMP_DIR}" ] && { errmsg "Missing -t option: no temporary directory."; }
	[ -z "${TPL_DIR}" ] && { errmsg "Missing -p option: no template directory."; }

	[ -d "${SRC_DIR}" ] || { errmsg "Bad -s option: source directory inaccessible."; }
	[ -d "${DST_DIR}" ] || { errmsg "Bad -d option: destination directory unusable."; }
	[ -d "${TMP_DIR}" ] || { errmsg "Bad -t option: temporary directory unusable."; }
	[ -d "${TPL_DIR}" ] || { errmsg "Bad -p option: template directory inaccessible."; }

	maketempdir

	#
	# Main loop preparation.
	#
	build="$(
		printf "%s %s" \
		"bin/txt2html.sh -s \"\${SRC_DIR}\" -d \"\${DST_DIR}\" -t \"\${TMP_DIR}\" -i \"\${TPL_DIR}\"" \
		"-x \"\${XLAT}\" -1 \"\${XLAT_OPTS1}\" -2 \"\${XLAT_OPTS2}\" -3 \"\${XLAT_OPTS3}\" -f"
		)"
	count=0
}


#
# Create temporary scratch directory and initialize temporary scratch files.
#
maketempdir() {
	TMP_WORK="$( mktemp -d "${TMP_DIR}/makefiles.XXXXXX" )" || { errmsg "mktemp err: '${TMP_WORK}'"; }

	touch "${TMP_WORK}/filelist"
	touch "${TMP_WORK}/badlist"
	touch "${TMP_WORK}/alltarget"
	touch "${TMP_WORK}/septargets"
	touch "${TMP_WORK}/header"
	touch "${TMP_WORK}/template"
	touch "${TMP_WORK}/dependencies"
	touch "${TMP_WORK}/deps"
	touch "${TMP_WORK}/files.mk"
}


#
# Remove temporary scratch directory and its temporary scratch files.
#
cleantempdir() {
	rmfile "${TMP_WORK}/filelist"
	rmfile "${TMP_WORK}/badlist"
	rmfile "${TMP_WORK}/alltarget"
	rmfile "${TMP_WORK}/septargets"
	rmfile "${TMP_WORK}/header"
	rmfile "${TMP_WORK}/template"
	rmfile "${TMP_WORK}/dependencies"
	rmfile "${TMP_WORK}/deps"
	rmfile "${TMP_WORK}/files.mk"

	rmdirectory "${TMP_WORK}"
}



#
# Process the file given in the variable ${file}.
#
processfile() {
	#
	# Discover this file's template name.
	#
	sed -n "1,/^$/p" "${file}" > "${TMP_WORK}/header"
	grep -e "^%tpl " "${TMP_WORK}/header" | sed -n "s/^%tpl //;p" > "${TMP_WORK}/template"
	TEMPLATE="$( tr -d "\n" < "${TMP_WORK}/template" )"
	if [ -z "${TEMPLATE}" ]
	then TEMPLATE="page.m4"; printf "%s\t%s\n" "${count}" "${DST_DIR}/${tname}"
	else printf "%s\t%s\t%s\n" "${count}" "${DST_DIR}/${tname}" "${TEMPLATE}"
	fi

	# Add the to file to the targets list.
	#
	tfname="\${DST_DIR}/${tname}"

	# Special case:  The sitemap page depends on sitemap.m4 in the same directory as the Makefile.
	# Make sure it's added as a prerequisite.
	#
	if [ "sitemap/index.html" = "${tname}" ]
	then sfname="\${SRC_DIR}/${fname} \${TPL_DIR}/${TEMPLATE} sitemap.m4"
	else sfname="\${SRC_DIR}/${fname} \${TPL_DIR}/${TEMPLATE}"
	fi

	#
	# Discover named dependencies and add them to the list behind the page template dependency.
	# (This is where the "%dep" source header option is processed.)
	#
	sed -n "1,/^$/p" "${file}" > "${TMP_WORK}/dependencies"
	grep -e "^%dep " "${TMP_WORK}/dependencies" | sed -n "s/^%dep //;p" > "${TMP_WORK}/deps"
	DEPS="$( tr "\n" " " < "${TMP_WORK}/deps" )"

	if [ -n "${DEPS}" ]; then processdeps; fi

	# Print out the makefile lines for the current target: one for the page target (alltarget), the other for
	# the individual target file and sources (septargets).
	#
	printf "%s\n" \
		"pages: ${tfname}" \
		>> "${TMP_WORK}/alltarget"
	printf "%s\n\t%s\n\n" \
		"${tfname}: ${sfname}" \
		"${build} \"${SRC_DIR}/${fname}\"" \
		>> "${TMP_WORK}/septargets"
}


#
# Process the list of dependency files given in the variable ${DEPS}.
# Called from processfile() above.
#
processdeps() {
	#
	# No spaces can be allowed in filenames here.
	#
	OLDIFS="$IFS"
	IFS="$( printf "%b_" " \t\n" )"; IFS="${IFS%_}"

	#
	# The list of dependencies is a list of dependency files to search for.  For each file in that list,
	# search the following locations for the file and used the first file found.  No subdirectories of these
	# directories are searched.  If the file isn't found, then don't add it, but print a warning instead.
	#
	# * The directory the source file is in
	# * The template directory TPL_DIR
	# * The source directory SRC_DIR
	# * Each directory between the source directory and the directory the source file is in, deepest first
	#
	# (Yes, this looks in SRC_DIR and the file's directory twice.)
	#
	deplist="${DEPS}"
	finaldeplist=""
	for depfile in ${deplist}
	do
		depf="${file}"
		depfi="${depfile}"

		# Compose list of directories to search when finding the dependency file.
		#
		depdirs="$( dirname "${depf}" )"
		depdirs="${depdirs} ${TPL_DIR} ${SRC_DIR}"
		founddep=""

		while [ "." != "${depf}" ]
		do
			depf="$( dirname "${depf}" )"
			if [ "." != "${depf}" ]; then depdirs="${depdirs} ${depf}"; fi
		done

		# Is the dependency file given as a path, and does it exist there?
		if [ -f "${depfi}" ]; then founddep="${depfile}"; fi

		# Look in the list of directories, and use the first file found.
		for depsearch in ${depdirs}
		do
			depfind="${depsearch}/${depfile}"
			if [ -f "${depfind}" ] && [ -z "${founddep}" ]; then founddep="${depfind}"; fi
		done

		# Add the dependency file for make, or show a warning message.
		if [ -n "${founddep}" ]
		then
			#
			# Undo the expansion of the source and template directory name variables.
			#
			founddep="$( printf "%s\n" "${founddep}" | sed -n "s/^${TPL_DIR}\//\$\{TPL_DIR\}\//;p" )"
			founddep="$( printf "%s\n" "${founddep}" | sed -n "s/^${SRC_DIR}\//\$\{SRC_DIR\}\//;p" )"

			finaldeplist="${finaldeplist} ${founddep}"
		else
			printf "%s\t%s\n" "--->" "WARNING: Dependency '${depfile}' was not found."
		fi
	done

	#
	# Trim leading and trailing spaces.
	#
	finaldeplist="$( printf "%s" "${finaldeplist}" | sed "s/^[ ]*//;s/[ ]*$//" )"
	sfname="${sfname} ${finaldeplist}"

	IFS="${OLDIFS}"
}


#
# Program proper start.
#
# Process command line options and initialize scratch space.
#
setoptions "$@"

#
# Find and save source *.txt file names.
#
# Skip over any files with whitespace in its name because make is hard-coded to treat all whitespace as delimiters, and it's
# impossible to override this behavior in any make-based build system.  The user needs to be shown what files were skipped.
# Processed files are in filelist, and skipped files are in badlist.
#
find -L "${SRC_DIR}/" -name "*.txt" -type f | sort -f | grep -v -e "[[:space:]]" > "${TMP_WORK}/filelist"
find -L "${SRC_DIR}/" -name "*.txt" -type f | sort -f | grep -e "[[:space:]]"    > "${TMP_WORK}/badlist"

#
# Process the filename targets that will be built.
#
printf "\n    %s\n" "File targets found:"

LOOPIFS="${IFS}"; IFS=""
while read -r file
do
	count=$(( count + 1 ))
	fname="$( printf "%s" "${file}" | sed -n "s:${SRC_DIR}/::;p" )"  	# From-name
	tname="$( printf "%s" "${fname}" | sed "s/\.txt$//;s/$/.html/" )"	# To-name
	processfile
done < "${TMP_WORK}/filelist"
IFS="${LOOPIFS}"

#
# Show the filename targets that won't be built.
#
if [ -s "${TMP_WORK}/badlist" ]
then
	printf "\n    %s\n" "Source files found but will not be built due to whitespace in file name:"
	count=0
	LOOPIFS="${IFS}"; IFS=""
	while read -r badname
	do
		if [ -n "${badname}" ]
		then
			count=$(( count + 1 ))
			printf "%s\t%s\n" "${count}" "${badname}"
		fi
	done < "${TMP_WORK}/badlist"
	IFS="${LOOPIFS}"
fi

#
# Combine alltarget and septargets into files.mk so it can be included by Makefile.
#
printf "%s\n%s\n%s\n%s\n%s\n%s\n\n%s\n\n\n%s\n\n\n" \
	"#" \
	"# Make targets for each source file." \
	"#" \
	"# DO NOT EDIT THIS FILE!" \
	"# This file is overwritten by the \"files\" make target." \
	"#" \
	"$( cat "${TMP_WORK}/alltarget" )" \
	"$( cat "${TMP_WORK}/septargets" )" \
	> "${TMP_WORK}/files.mk"

if [ -s "${TMP_WORK}/files.mk" ]
then cp -f -- "${TMP_WORK}/files.mk" "files.mk"
else printf "%s\t%s\n" "--->" "ERROR:  The replacement files.mk file could not be written to."; exit 1
fi

printf "\n    %s\n" "File targets made.  Make the \"pages\" target to build them."

#
# All work done, clean up scratch space.
#
cleantempdir

printf "%s\n" "done."
# End.

