#!/bin/sh

# Terminal width: 132 columns.
# Function definitions end at comment "# Program proper start."

#
# Print usage and exit.
#
usage() {
	usageflag="$1"
	usageself="$( basename "$0" )"
	usagemessage="$( cat <<- HERE
	Create and populate directories for a new static website.

	Usage:

	${usageself} -h|-?
	${usageself} -s src_dir -d dest_dir -l tpl_dir -p pub_dir -t temp_dir

	-h or -?     Show this help text and exit without error.
	-s src_dir   Directory src_dir containing files to convert to HTML pages.
	-d dest_dir  Directory dest_dir containing the converted and staged HTML pages.
	-l tpl_dir   Directory tpl_dir containing template files for M4 conversion.
	-p pub_dir   Directory pub_dir containing public no-compile asset files.
	-t temp_dir  Directory temp_dir to use for temporary scratch files.

	Directory options are either absolute paths or relative to the current
	directory, not the directory in which ${usageself} lives.
	HERE
	)"
	if [ -n "${usageflag}" ]
	then printf "%s\n" "${usagemessage}"; exit 0
	else printf "%s\n" "${usagemessage}" >&2; exit 1
	fi
}


#
# Print an error message, then print usage and exit.
#
errmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" | fold -s >&2
	printf "%s\n" "Edit 'config.mk' as needed to fix this error." >&2
	exit 1
}


#
# Process command line options and initialize variables.
#
setoptions() {
	unset SRC_DIR DST_DIR TPL_DIR PUB_DIR TMP_DIR

	while getopts 's:d:l:p:t:h?' c
	do
		case $c in
		s)  SRC_DIR="${OPTARG}"  ;;
		d)  DST_DIR="${OPTARG}"  ;;
		l)  TPL_DIR="${OPTARG}"  ;;
		p)  PUB_DIR="${OPTARG}"  ;;
		t)  TMP_DIR="${OPTARG}"  ;;
		h)  usage help           ;;
		?)  usage help           ;;
		esac
	done

	printf "%s %s\n" \
		"Checking that SRC_DIR, DST_DIR, TPL_DIR, PUB_DIR, and TMP_DIR are set in 'config.mk'," \
		"don't refer to directories that already exist, and don't refer to things that aren't directories." \
		| fold -s

	[ -z "${SRC_DIR}" ] && { errmsg "SRC_DIR empty.  Set SRC_DIR = directory with unconverted source files."; }
	[ -z "${DST_DIR}" ] && { errmsg "DST_DIR empty.  Set DST_DIR = directory to hold the staged website's files."; }
	[ -z "${TPL_DIR}" ] && { errmsg "TPL_DIR empty.  Set TPL_DIR = directory with the website's template files."; }
	[ -z "${PUB_DIR}" ] && { errmsg "PUB_DIR empty.  Set PUB_DIR = directory with public data files without conversion."; }
	[ -z "${TMP_DIR}" ] && { errmsg "TMP_DIR empty.  Set TMP_DIR = directory to use for temporary files in the build process."; }
}


#
# Program proper start.
#
# Process command line options and initialize scratch space.
#
setoptions "$@"


#
# Check that each directory exists.  If not, make it and populate it.
# If it exists but isn't a directory, show an error message and quit.
#
if [ -d "bin/" ] && [ -d "bin/template_src" ] && [ -d "bin/template_tpl" ] && [ -d "bin/template_pub" ]
then
	:
else
	printf "%s\n" \
		"The binaries directory 'bin/' is misplaced.  It needs to be a subdirectory of the present working directory." \
		| fold -s
	exit 1
fi

#
# SRC_DIR: Website source files.
#
if [ -e "${SRC_DIR}/" ] && [ -d "${SRC_DIR}/" ]
then printf "%s\n" "Skipping SRC_DIR directory '${SRC_DIR}/' because it already exists." | fold -s
fi
if [ -e "${SRC_DIR}/" ] && [ ! -d "${SRC_DIR}/" ]
then
	printf "%s  %s\n" \
		"SRC_DIR directory '${SRC_DIR}' already exists but not as a directory." \
		"Edit 'config.mk' and set SRC_DIR to a directory name." \
		| fold -s
	exit 1
fi
if [ ! -e "${SRC_DIR}/" ]
then
	printf "%s" "Creating SRC_DIR directory '${SRC_DIR}/' ... "
	if mkdir -p -- "${SRC_DIR}/"
	then
		printf "%s" "populating ... "
		if cp -a -f -- "bin/template_src"/* "${SRC_DIR}"
		then printf "%s\n" "done."
		else printf "%s\n" "failed."
		fi
	else
		printf "%s\n" "failed."
	fi
fi


#
# DST_DIR: Website staging area.
#
if [ -e "${DST_DIR}/" ] && [ -d "${DST_DIR}/" ]
then printf "%s\n" "Skipping DST_DIR directory '${DST_DIR}/' because it already exists." | fold -s
fi
if [ -e "${DST_DIR}/" ] && [ ! -d "${DST_DIR}/" ]
then
	printf "%s  %s\n" \
		"DST_DIR directory '${DST_DIR}' already exists but not as a directory." \
		"Edit 'config.mk' and set DST_DIR to a directory name." \
		| fold -s
	exit 1
fi
if [ ! -e "${DST_DIR}/" ]
then
	printf "%s" "Creating DST_DIR directory '${DST_DIR}/' ... "
	if mkdir -p -- "${DST_DIR}/"
	then
		printf "%s\n" "done."
	else
		printf "%s\n" "failed."
	fi
fi


#
# TPL_DIR: Website page templates and M4 helpers.
#
if [ -e "${TPL_DIR}/" ] && [ -d "${TPL_DIR}/" ]
then printf "%s\n" "Skipping TPL_DIR directory '${TPL_DIR}/' because it already exists." | fold -s
fi
if [ -e "${TPL_DIR}/" ] && [ ! -d "${TPL_DIR}/" ]
then
	printf "%s  %s\n" \
		"TPL_DIR directory '${TPL_DIR}' already exists but not as a directory." \
		"Edit 'config.mk' and set TPL_DIR to a directory name." \
		| fold -s
	exit 1
fi
if [ ! -e "${TPL_DIR}/" ]
then
	printf "%s" "Creating TPL_DIR directory '${TPL_DIR}/' ... "
	if mkdir -p -- "${TPL_DIR}/"
	then
		printf "%s" "populating ... "
		if cp -a -f -- "bin/template_tpl"/* "${TPL_DIR}"
		then printf "%s\n" "done."
		else printf "%s\n" "failed."
		fi
	else
		printf "%s\n" "failed."
	fi
fi


#
# PUB_DIR: Website static files, includes, and other assets.
#
if [ -e "${PUB_DIR}/" ] && [ -d "${PUB_DIR}/" ]
then printf "%s\n" "Skipping PUB_DIR directory '${PUB_DIR}/' because it already exists." | fold -s
fi
if [ -e "${PUB_DIR}/" ] && [ ! -d "${PUB_DIR}/" ]
then
	printf "%s  %s\n" \
		"PUB_DIR directory '${PUB_DIR}' already exists but not as a directory." \
		"Edit 'config.mk' and set PUB_DIR to a directory name." \
		| fold -s
	exit 1
fi
if [ ! -e "${PUB_DIR}/" ]
then
	printf "%s" "Creating PUB_DIR directory '${PUB_DIR}/' ... "
	if mkdir -p -- "${PUB_DIR}/"
	then
		printf "%s" "populating ... "
		if cp -a -f -- "bin/template_pub"/* "${PUB_DIR}"
		then printf "%s\n" "done."
		else printf "%s\n" "failed."
		fi
	else
		printf "%s\n" "failed."
	fi
fi


#
# TMP_DIR: Website building scratch pads.
#
if [ -e "${TMP_DIR}/" ] && [ -d "${TMP_DIR}/" ]
then printf "%s\n" "Skipping TMP_DIR directory '${TMP_DIR}/' because it already exists." | fold -s
fi
if [ -e "${TMP_DIR}/" ] && [ ! -d "${TMP_DIR}/" ]
then
	printf "%s  %s\n" \
		"TMP_DIR directory '${TMP_DIR}' already exists but not as a directory." \
		"Edit 'config.mk' and set TMP_DIR to a directory name." \
		| fold -s
	exit 1
fi
if [ ! -e "${TMP_DIR}/" ]
then
	printf "%s" "Creating TMP_DIR directory '${TMP_DIR}/' ... "
	if mkdir -p -- "${TMP_DIR}/"
	then printf "%s\n" "done."
	else printf "%s\n" "failed."
	fi
fi


#
# All work done.
#
printf "%s\n" "done."

# End.

