#!/bin/sh

# Terminal width: 132 columns.
# Function definitions end at comment "# Program proper start."

#
# Print usage and exit.
#
usage() {
	usageflag="$1"
	usageself="$( basename "$0" )"
	usagemessage="$( cat <<- HERE
	Pluck and archive the AMTSWG files out of the current website for use in a
	new website.

	Usage:

	${usageself} -h|-?
	${usageself} -t temp_dir -f archive

	-h or -?     Show this help text and exit without error.
	-t temp_dir  Directory to use for repackaging the archive file.
	-f archive   Tar file archive to hold the plucked files.

	Directory paths are either absolute paths or relative to the current
	directory, not the directory in which ${usageself} lives.
	HERE
	)"
	if [ -n "${usageflag}" ]
	then printf "%s\n" "${usagemessage}"; exit 0
	else printf "%s\n" "${usagemessage}" >&2; exit 1
	fi
}


#
# Process command line options and initialize variables.
#
setoptions() {
	unset PLUCK

	while getopts 't:f:h?' c
	do
		case $c in
		t)  TMP_DIR="${OPTARG}"  ;;
		f)  PLUCK="${OPTARG}"    ;;
		h)  usage help           ;;
		?)  usage help           ;;
		esac
	done

	[ -z "${TMP_DIR}" ] && { errmsg "Missing -t option: no temporary directory."; }
	[ -z "${PLUCK}"   ] && { errmsg "Missing -f option: no archive filename."; }

	[ -d "${TMP_DIR}" ] || { errmsg "Bad -t option: temporary directory unusable."; }
	[ -e "${PLUCK}"   ] && { errmsg "Bad -f option: archive filename already exists."; }

	printf "%s" "Initializing ... "
	maketempdir
	list
}


#
# The list of files to be plucked.
#
list() {
	printf "%s\n" "
CHANGES
CONTRIBUTORS
LICENSE
Makefile
README
README.more
README.robotstxt
VERSION
bin/check.sh
bin/fixmakefile.sh
bin/help.sh
bin/makefiles.sh
bin/makestrptime.sh
bin/markdown.sh
bin/new.sh
bin/pluck.sh
bin/rickdate.sh
bin/sitemap.sh
bin/strptime.c
bin/syncpub.sh
bin/template_pub/avatar.png
bin/template_pub/buttons/lynxfriendly.jpg
bin/template_pub/buttons/valid-css-blue.gif
bin/template_pub/buttons/valid-css-blue.png
bin/template_pub/buttons/valid-css-gold.gif
bin/template_pub/buttons/valid-css-gold.png
bin/template_pub/buttons/valid-html-blue.png
bin/template_pub/buttons/valid-html40-blue.png
bin/template_pub/buttons/valid-html40-gold.png
bin/template_pub/buttons/valid-xhtml10-blue.png
bin/template_pub/buttons/valid-xhtml10-gold.png
bin/template_pub/buttons/wcag20aaa-gold.png
bin/template_pub/buttons/wcag21aaa-blue.png
bin/template_pub/buttons/wcag21aaa-gold.png
bin/template_pub/css/basic/box-clipart-png-sm.png
bin/template_pub/css/basic/normalize.css
bin/template_pub/css/basic/sitelogo.css
bin/template_pub/css/basic/sitelogo.css.dist
bin/template_pub/css/basic.css
bin/template_pub/css/greenbar-code.css
bin/template_pub/css/parchment/flourish-9.png
bin/template_pub/css/parchment/horizontal-scroll.png
bin/template_pub/css/parchment/lightparchment.png
bin/template_pub/css/parchment/normalize.css
bin/template_pub/css/parchment/rolled-parchment.png
bin/template_pub/css/parchment/tabletop.jpg
bin/template_pub/css/parchment/yeoldscroll.png
bin/template_pub/css/parchment.css
bin/template_pub/css/pygments/algol.css
bin/template_pub/css/pygments/algol_wcag2aaa.css
bin/template_pub/css/pygments/rouge-base16-dark.css
bin/template_pub/css/time-machine/code.png
bin/template_pub/css/time-machine/normalize.css
bin/template_pub/css/time-machine/pattern.png
bin/template_pub/css/time-machine/tar.png
bin/template_pub/css/time-machine/top.png
bin/template_pub/css/time-machine/zip.png
bin/template_pub/css/time-machine.css
bin/template_pub/favicon/packing-box-114.png
bin/template_pub/favicon/packing-box-120.png
bin/template_pub/favicon/packing-box-144.png
bin/template_pub/favicon/packing-box-152.png
bin/template_pub/favicon/packing-box-16.png
bin/template_pub/favicon/packing-box-24.png
bin/template_pub/favicon/packing-box-32.png
bin/template_pub/favicon/packing-box-48.png
bin/template_pub/favicon/packing-box-57.png
bin/template_pub/favicon/packing-box-64.png
bin/template_pub/favicon/packing-box-72.png
bin/template_pub/favicon/packing-box-favicon.ico
bin/template_pub/favicon.ico
bin/template_pub/favicon.ico.dist
bin/template_src/index.txt
bin/template_src/index.txt.sample
bin/template_tpl/docutils-full.tpl
bin/template_tpl/highlight.m4
bin/template_tpl/htmlhelpers.m4
bin/template_tpl/page.m4.basic
bin/template_tpl/page.m4.parchment
bin/template_tpl/page.m4.timemachine
bin/template_tpl/pygment.m4
bin/template_tpl/rst2html.tpl
bin/txt2html.sh
bin/upload.sh
bin/webtest.sh
config.mk.sample
dateformat
files.mk.if-make-aborts
sitemap.m4.markdown
sitemap.m4.raw-html
sitemap.m4.restructuredtext
sitemap.m4.txt2tags
" \
	| sed "/^$/d" \
	> "${TMP_WORK}/list"
}


#
# Print an error message, then print usage and exit.
#
errmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" >&2
	usage
}


#
# Remove a file or show an error message.
#
rmfile() {
	if [ -z "$1" ]; then return; fi
	if [ -e "$1" ]; then rm -f -- "$1" || { errmsg "Can't rm file '$1'."; }; else errmsg "File to rm '$1' doesn't exist."; fi
}


#
# Remove an empty directory or show an error message.
#
rmdirectory() {
	if [ -z "$1" ]; then return; fi

	if [ -d "$1" ]
	then rmdir -- "$1" || { errmsg "Can't rmdir directory '$1'."; }
	else errmsg "Directory to rmdir '$1' doesn't exist."
	fi
}


#
# Create temporary scratch directory and initialize temporary scratch files.
#
maketempdir() {
	TMP_WORK="$( mktemp -d -- "${TMP_DIR}/pluck.XXXXXX" )" || { errmsg "mktemp err: '${TMP_WORK}'"; }

	mkdir -- "${TMP_WORK}/amtswg" || { errmsg "Can't mkdir directory '${TMP_WORK}/amtswg'."; }

	touch "${TMP_WORK}/list"
	touch "${TMP_WORK}/plucked.tar"
	touch "${TMP_WORK}/amtswg.tar"
}


#
# Remove temporary scratch directory and its temporary scratch files.
#
cleantempdir() {
	rm -r -f -- "${TMP_WORK}/amtswg"/* || { errmsg "Can't rm -r -f -- '${TMP_WORK}/amtswg/*'."; }
	rmdirectory "${TMP_WORK}/amtswg"

	rmfile "${TMP_WORK}/list"
	rmfile "${TMP_WORK}/plucked.tar"
	rmfile "${TMP_WORK}/amtswg.tar"

	rmdirectory "${TMP_WORK}"
}


#
# Program proper start.
#
# Process command line options.
#
setoptions "$@"


#
# The plucked files need to live in a directory called "amtswg/", but they're being plucked from the archive's root directory.
# In order to have them packed properly in a subdirectory, they have to be unpacked into that subdirectory then repacked from
# that subdirectory's parent directory, making this a four-step process.
#

# 1.  Pluck the archive out of the live website.
#
printf "%s" "plucking ... "
pax -w -x ustar -f "${TMP_WORK}/plucked.tar" < "${TMP_WORK}/list"

# 2.  Unpack the plucked archive into the repacking directory.
#
(
	printf "%s" "unpacking ... "
	cd "${TMP_WORK}/amtswg"
	pax -r -f "../plucked.tar"
)


# 3.  Repack the archive with the right parent directory.
#
(
	printf "%s" "repacking ... "
	cd "${TMP_WORK}"
	pax -w -x ustar -f "amtswg.tar" amtswg/
)

# 4.  Put the repacked archive where the user wants it.
#
printf "%s" "relocating ... "
cp -f -- "${TMP_WORK}/amtswg.tar" "${PLUCK}"


#
# All work done, clean up scratch space.
#
cleantempdir

printf "%s\n" "done."
# End.

