#!/bin/sh

# Terminal width: 132 columns.
# Function definitions end at comment "# Program proper start."

#
# Print usage and exit.
#
usage() {
	usageflag="$1"
	usageself="$( basename "$0" )"
	usagemessage="$( cat <<- HERE
	Create sitemap source page for visitors in [src_dir]/sitemap/index.txt and
	search engine sitemap file [pub_dir]/sitemap.txt.

	Usage:

	${usageself} -h|-?
	${usageself} -s src_dir -t temp_dir -p pub_dir -m title -u urlbase

	-h or -?     Show this help text and exit without error.
	-s src_dir   Directory src_dir containing /sitemap/index.txt* files.
	-t temp_dir  Directory temp_dir to use for temporary scratch files.
	-p pub_dir   Directory pub_dir containing sitemap.org /sitemap.txt file.
	-m title     Title of the sitemap page visitors will see.
	-u urlbase   Base URL of the published website.

	Directory options are either absolute paths or relative to the current
	directory, not the directory in which ${usageself} lives.
	HERE
	)"
	if [ -n "${usageflag}" ]
	then printf "%s\n" "${usagemessage}"; exit 0
	else printf "%s\n" "${usagemessage}" >&2; exit 1
	fi
}


#
# Print an error message, then print usage and exit.
#
errmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" >&2
	usage
}


#
# Remove a file or show an error message.
#
rmfile() {
	if [ -z "$1" ]; then return; fi
	if [ -e "$1" ]; then rm -f -- "$1" || { errmsg "Can't rm file '$1'."; }; else errmsg "File to rm '$1' doesn't exist."; fi
}


#
# Remove an empty directory or show an error message.
#
rmdirectory() {
	if [ -z "$1" ]; then return; fi

	if [ -d "$1" ]
	then rmdir -- "$1" || { errmsg "Can't rmdir directory '$1'."; }
	else errmsg "Directory to rmdir '$1' doesn't exist."
	fi
}


#
# Process command line options and initialize variables.
#
setoptions() {
	unset SRC_DIR TMP_DIR PUB_DIR TITLE URLBASE

	while getopts 's:t:p:m:u:h?' c
	do
		case $c in
		s)  SRC_DIR="${OPTARG}"  ;;
		t)  TMP_DIR="${OPTARG}"  ;;
		p)  PUB_DIR="${OPTARG}"  ;;
		m)  TITLE="${OPTARG}"    ;;
		u)  URLBASE="${OPTARG}"  ;;
		h)  usage help           ;;
		?)  usage help           ;;
		esac
	done

	[ -z "${SRC_DIR}" ] && { errmsg "Missing -s option: no source directory."; }
	[ -z "${TMP_DIR}" ] && { errmsg "Missing -t option: no temporary directory."; }
	[ -z "${PUB_DIR}" ] && { errmsg "Missing -p option: no public directory."; }
	[ -z "${TITLE}"   ] && { errmsg "Missing -m option: no page title."; }
	[ -z "${URLBASE}" ] && { errmsg "Missing -u option: no website base URL."; }

	[ -d "${SRC_DIR}" ] || { errmsg "Bad -s option: source directory inaccessible."; }
	[ -d "${TMP_DIR}" ] || { errmsg "Bad -t option: temporary directory unusable."; }
	[ -d "${PUB_DIR}" ] || { errmsg "Bad -p option: public directory unusable."; }

	maketempdir

	SITEMAP="${TMP_WORK}/sitemap"
	SITEMAPFINAL="${SRC_DIR}/sitemap/index.txt"
	SITEMAPORG="${TMP_WORK}/sitemaporg"
	SITEMAPORGFINAL="${PUB_DIR}/sitemap.txt"
}


#
# Create temporary scratch directory and initialize temporary scratch files.
#
maketempdir() {
	TMP_WORK="$( mktemp -d "${TMP_DIR}/sitemap.XXXXXX" )" || { errmsg "mktemp err: '${TMP_WORK}'"; }

	touch "${TMP_WORK}/header"
	touch "${TMP_WORK}/sitemap"
	touch "${TMP_WORK}/sitemaporg"
	touch "${TMP_WORK}/filelist"
}


#
# Remove temporary scratch directory and its temporary scratch files.
#
cleantempdir() {
	rmfile "${TMP_WORK}/header"
	rmfile "${TMP_WORK}/sitemap"
	rmfile "${TMP_WORK}/sitemaporg"
	rmfile "${TMP_WORK}/filelist"

	rmdirectory "${TMP_WORK}"
}


#
# Rotate in the new sitemaps.
#
savesitemaps() {
	#
	# Visitor sitemap page source.
	#
	# Make sure directory exists (it won't on the first ever run).
	mkdir -p "${SRC_DIR}/sitemap"
	if [ ! -d "${SRC_DIR}/sitemap" ]; then errmsg "Couldn't mkdir '${SRC_DIR}/sitemap'.  TMP_WORK: ${TMP_WORK}"; fi

	# Rotate existing files.
	rm -f -- "${SITEMAPFINAL}.99"
	j=99; while [ 0 -lt $j ]
	do
		i=$((j - 1))
		if [ -f "${SITEMAPFINAL}.${i}" ] && [ ! -f "${SITEMAPFINAL}.${j}" ]
		then mv -- "${SITEMAPFINAL}.${i}" "${SITEMAPFINAL}.${j}"
		fi
	j=$((j - 1))
	done
	if [ -f "${SITEMAPFINAL}" ] && [ ! -f "${SITEMAPFINAL}.0" ]
	then mv -- "${SITEMAPFINAL}" "${SITEMAPFINAL}.0"
	fi

	# Bring in new file.
	if [ ! -f "${SITEMAPFINAL}" ]
	then cp -f -- "${SITEMAP}" "${SITEMAPFINAL}"
	fi

	#
	# Sitemap.org sitemap list.
	#
	# Not rotated, so just bring it in if it's ready.
	cp -f -- "${SITEMAPORG}" "${SITEMAPORGFINAL}"
}

#
# Program proper start.
#
# Process command line options and initialize scratch space.
#
setoptions "$@"

printf "    %s\n" "Generating sitemap:"

#
# Write header and headings of visitor sitemap page.
#
printf "%s\n%s\n%s\n\n%s\n%s\n%s\n%s\n" \
	"%t ${TITLE}" \
	"%a ArielMT's Static Website Generator (AMTSWG)" \
	"%d $( date +"%A, %B %e, %Y" )" \
	\
	"m4_changequote([[\`]],[[']])m4_dnl" \
	"m4_include(\`sitemap.m4')m4_dnl" \
	"PRE" \
	"OPEN" \
	>> "${SITEMAP}"

#
# Loop over every file in the source directory except the sitemap, skip over any files not source page files, and put "index.txt"
# files at the head of each directory's listing.
#
# ("Put 'index.txt' files at the head of each directory's listing" means linking to them [or their directory-name shortcuts] before
# any other files in the directory.  They would otherwise end up sorted into the middle of the list of files.)
#
(
	find -L "${SRC_DIR}/" \
	-name "index.txt" \
	\! \( -path "${SRC_DIR}*/.*" \) \
	\! \( -path "${SITEMAPFINAL%/index.txt}*" \) \
	-print | sed "s:index.txt$::"
	find -L "${SRC_DIR}/" \
	-name "*.txt" \
	-type f \
	\! \( -path "${SRC_DIR}*/.*" \) \
	\! \( -name "index.txt" \) \
	-print
) | sort -f | sed "s:/$:/index.txt:" | grep -v -e "[[:space:]]" >> "${TMP_WORK}/filelist"

count=0
LOOPIFS="${IFS}"; IFS=""
while read -r file # done < "${TMP_WORK}/filelist"
do
	count=$((count + 1))
	fname="$( printf "%s" "${file}" | sed -n "s:${SRC_DIR}/::;p" )"                    	# From-name
	tname="$( printf "%s" "${fname}" | sed "s/\.txt$//;s/$/.html/;s:/index.html$:/:" )"	# To-name
	# If the to-file's basename is "index.html", then treat it specially because links to it are most often links to the
	# directory name without the filename.  (last sed in code above.)

	# Special case: "${SRC_DIR}/index.txt" becomes "index.html" without a leading slash.  Make it "/" instead.
	if [ "index.html" = "${tname}" ]; then tname=""; fi

	# Discover page title.
	#
	sed -n "1,/^$/p" "${file}" > "${TMP_WORK}/header"
	TITLE="$( grep -e "^%t " "${TMP_WORK}/header" | sed -n "s/^%t //;p" | tr -d "\n" )"
	if [ -z "${TITLE}" ]; then TITLE="$( basename "${tname}" )"; fi
	tsname="${tname}"; if [ -z "${tsname}" ]; then tsname="/"; fi
	if [ "$( basename "${tname}" )" = "${TITLE}" ]
	then printf "%s\t%s\n" "${count}" "${tsname}"
	else printf "%s\t%s\n\t\t%s\n" "${count}" "${tsname}" "${TITLE}"
	fi

	printf "%s\n" "${URLBASE}${tname}" >> "${SITEMAPORG}"

	# Bug: Apostrophes kept getting pushed to end of title string.
	# Bugfix: Forgot that apostrophes can't be escaped in M4, so translate apostrophes to the HTML "&apos;" entity.
	TITLE="$( printf "%s" "${TITLE}" | sed -n "s/\'/\&apos;/g;p" )"

	if [ -z "${tname}" ]; then DIR="."; else DIR="$( dirname "${tname}" )"; fi
	smline="ITEM(\`${count}',\`"
	if [ "." != "${DIR}" ]; then smline="${smline}${DIR}: "; fi
	smline="${smline}LINK(\`${count}',\`/${tname}',\`${TITLE}')')"
	printf "%s\n" "${smline}" >> "${SITEMAP}"

done < "${TMP_WORK}/filelist" # while read -r file
IFS="${LOOPIFS}"

#
# Close out sitemap
#
printf "%s\n\n" "CLOSE" >> "${SITEMAP}"

savesitemaps

#
# All work done, clean up scratch space.
#
cleantempdir

printf "%s\n" "done."
# End.

