#!/bin/sh

# Terminal width: 132 columns.
# Function definitions end at comment "# Program proper start."

#
# Print usage and exit.
#
usage() {
	usageflag="$1"
	usageself="$( basename "$0" )"
	usagemessage="$( cat <<- HERE
	Synchronize staging directory's contents with those of source and public.

	Usage:

	${usageself} -h|-?
	${usageself} -s src_dir -d dest_dir -t temp_dir -p pub_dir

	-h or -?     Show this help text and exit without error.
	-s src_dir   Directory src_dir containing files to convert to HTML pages.
	-d dest_dir  Directory dest_dir containing the converted and staged HTML pages.
	-t temp_dir  Directory temp_dir to use for temporary scratch files.
	-p pub_dir   Directory pub_dir containing public no-compile files.

	Directory options are either absolute paths or relative to the current
	directory, not the directory in which ${usageself} lives.
	HERE
	)"
	if [ -n "${usageflag}" ]
	then printf "%s\n" "${usagemessage}"; exit 0
	else printf "%s\n" "${usagemessage}" >&2; exit 1
	fi
}


#
# Print an error message, then print usage and exit.
#
errmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" >&2
	usage
}


#
# Print an error message, then exit.
#
abortmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" >&2
	exit 1
}
	

#
# Remove a file or show an error message.
#
rmfile() {
	if [ -z "$1" ]; then return; fi
	if [ -e "$1" ]; then rm -f -- "$1" || { errmsg "Can't rm file '$1'."; }; else errmsg "File to rm '$1' doesn't exist."; fi
}


#
# Remove an empty directory or show an error message.
#
rmdirectory() {
	if [ -z "$1" ]; then return; fi

	if [ -d "$1" ]
	then rmdir -- "$1" || { errmsg "Can't rmdir directory '$1'."; }
	else errmsg "Directory to rmdir '$1' doesn't exist."
	fi
}


#
# Process command line options and initialize variables.
#
setoptions() {
	unset SRC_DIR DST_DIR TMP_DIR PUB_DIR

	while getopts 's:d:t:p:h?' c
	do
		case $c in
		s)  SRC_DIR="${OPTARG}"  ;;
		d)  DST_DIR="${OPTARG}"  ;;
		t)  TMP_DIR="${OPTARG}"  ;;
		p)  PUB_DIR="${OPTARG}"  ;;
		h)  usage help           ;;
		?)  usage help           ;;
		esac
	done

	[ -z "${SRC_DIR}" ] && { errmsg "Missing -s option: no source directory."; }
	[ -z "${DST_DIR}" ] && { errmsg "Missing -d option: no destination directory."; }
	[ -z "${TMP_DIR}" ] && { errmsg "Missing -t option: no temporary directory."; }
	[ -z "${PUB_DIR}" ] && { errmsg "Missing -p option: no public directory."; }

	[ -d "${SRC_DIR}" ] || { errmsg "Bad -s option: source directory inaccessible."; }
	[ -d "${DST_DIR}" ] || { errmsg "Bad -d option: destination directory unusable."; }
	[ -d "${TMP_DIR}" ] || { errmsg "Bad -t option: temporary directory unusable."; }
	[ -d "${PUB_DIR}" ] || { errmsg "Bad -p option: public directory inaccessible."; }

	maketempdir

	printf "%s\n" \
		"Synchronizing '${DST_DIR}/' to match '${PUB_DIR}/' and pages generated from files in '${SRC_DIR}/':" | \
		fold -s
}


#
# Create temporary scratch directory and initialize temporary scratch files.
#
maketempdir() {
	TMP_WORK="$( mktemp -d "${TMP_DIR}/syncpub.XXXXXX" )" || { errmsg "mktemp err: '${TMP_WORK}'"; }

	touch "${TMP_WORK}/findsource"
	touch "${TMP_WORK}/findpub"
	touch "${TMP_WORK}/unsorted-source"
	touch "${TMP_WORK}/unsorted-dest"
	touch "${TMP_WORK}/sorted-source"
	touch "${TMP_WORK}/sorted-dest"
	touch "${TMP_WORK}/only-source"
	touch "${TMP_WORK}/only-dest"
	touch "${TMP_WORK}/common"
}


#
# Remove temporary scratch directory and its temporary scratch files.
#
cleantempdir() {
	rmfile "${TMP_WORK}/findsource"
	rmfile "${TMP_WORK}/findpub"
	rmfile "${TMP_WORK}/unsorted-source"
	rmfile "${TMP_WORK}/unsorted-dest"
	rmfile "${TMP_WORK}/sorted-source"
	rmfile "${TMP_WORK}/sorted-dest"
	rmfile "${TMP_WORK}/only-source"
	rmfile "${TMP_WORK}/only-dest"
	rmfile "${TMP_WORK}/common"

	rmdirectory "${TMP_WORK}"
}


#
# Program proper start.
#
# Process command line options and initialize scratch space.
#
setoptions "$@"

#
# Unwise in general, but spaces in filenames are okay here.  The files found will be copied, not processed.
#
IFS="$( printf "%b_" "\t\n" )"; IFS="${IFS%_}"

#
# Find files in SRC_DIR whose compiled forms should be in DST_DIR.
#
find -L "${SRC_DIR}/" -name "*.txt" -type f | grep -v -e "[[:space:]]" > "${TMP_WORK}/findsource"
while read -r file
do
	tname="$( printf "%s" "${file}" | sed -n "s:${SRC_DIR}/:${DST_DIR}/:;p" | sed "s/\.txt$//;s/$/.html/" )"
	printf "%s\n" "${tname}" >> "${TMP_WORK}/unsorted-source"
done < "${TMP_WORK}/findsource"

#
# Find files in PUB_DIR that should be in DST_DIR.
#
find -L "${PUB_DIR}/" -type f > "${TMP_WORK}/findpub"
while read -r file
do
	tname="$( printf "%s" "${file}" | sed -n "s:${PUB_DIR}/:${DST_DIR}/:;p" )"
	printf "%s\n" "${tname}" >> "${TMP_WORK}/unsorted-source"
done < "${TMP_WORK}/findpub"

#
# Find files in DST_DIR already, whether they should be there or not.
#
find -L "${DST_DIR}/" -type f > "${TMP_WORK}/unsorted-dest"

#
# Sort and separate lists.
#
sort "${TMP_WORK}/unsorted-source" > "${TMP_WORK}/sorted-source"
sort "${TMP_WORK}/unsorted-dest"   > "${TMP_WORK}/sorted-dest"
comm -12 "${TMP_WORK}/sorted-source" "${TMP_WORK}/sorted-dest" > "${TMP_WORK}/common"
comm -13 "${TMP_WORK}/sorted-source" "${TMP_WORK}/sorted-dest" > "${TMP_WORK}/only-dest"
comm -23 "${TMP_WORK}/sorted-source" "${TMP_WORK}/sorted-dest" > "${TMP_WORK}/only-source"

#
# Delete files in DST_DIR that shouldn't be there.
#
while read -r file
do
	printf "%s" "Deleting orphaned '${file}' ..."
	rm -f -- "${file}" || abortmsg "failed."
	rmdir -p -- "$( dirname "${file}" )" 2>/dev/null
	printf "%s\n" "done."
done < "${TMP_WORK}/only-dest"

#
# For every file that should be in DST_DIR but isn't, if it's in PUB_DIR, then copy it.
#
while read -r file
do
	fname="$( printf "%s" "${file}" | sed -n "s:${DST_DIR}/:${PUB_DIR}/:;p" )"
	if [ -f "${fname}" ]
	then
		printf "%s" "Copying missing '${file}' ..."
		mkdir -p -- "$( dirname "${file}" )"
		cp -af -- "${fname}" "${file}" || abortmsg "failed."
		printf "%s\n" "done."
	else
		printf "%s\n" "Did not copy missing '${file}'."
	fi
done < "${TMP_WORK}/only-source"

#
# For every file in DST_DIR that should be there, make sure it's up to date.
#
while read -r file
do
	fname="$( printf "%s" "${file}" | sed -n "s:${DST_DIR}/:${PUB_DIR}/:;p" )"
	# if [ "${fname}" -nt "${file}" ]
	# printf "%s\t%s\n" "'${fname}'" "'${file}'"
	if [ -n "$( find "${fname}" -prune -newer "${file}" 2>/dev/null )" ]
	then
		printf "%s" "Copying newer '${file}' ..."
		cp -af -- "${fname}" "${file}" || abortmsg "failed."
		printf "%s\n" "done."
	fi
done < "${TMP_WORK}/common"


#
# All work done, clean up scratch space.
#
cleantempdir

printf "%s\n" "done."
# End.

