#!/bin/sh

# Terminal width: 132 columns.
# function definitions end at comment "# Program proper start."

#
# Print usage and exit.
#
usage() {
	usageflag="$1"
	usageself=$( basename "$0" )
	usagemessage="$( cat <<- HERE
	Convert a source file from LML or HTML fragment to a staged HTML page.

	Usage:

	${usageself} -h|-?
	${usageself} -s src_dir -d dest_dir -t temp_dir -i tpl_dir
		-x command [ -1 options [ -2 options [ -3 options ] ] ] -f file

	-h or -?     Show this help text and exit without error.
	-s src_dir   Directory src_dir containing source file to convert.
	-d dest_dir  Directory dest_dir containing the converted and staged HTML page.
	-t temp_dir  Directory temp_dir to use for temporary scratch files.
	-i tpl_dir   Directory tpl_dir containing website templates.
	-x command   LML translation command to use.
	-1 options1  Translation options between command and input file names.
	-2 options2  Translation options between input and output file names.
	-3 options3  Translation options after output file name.
	-f file      Relative to src_dir, the file to convert from LML to HTML.

	When the command given by -x is invoked, the parameters given by -x, -1, -2,
	and -3 are assembled and executed as follows:

	command options1 inputfilename options2 outputfilename options3

	If no translation needs to be done (that is, the input file is already an HTML
	fragment), then the -x option should be "-x cp", the -1 option should be
	"-1 '-f'", and the -2 and -3 options should not be provided.

	Directory options are either absolute paths or paths relative to the current
	directory, not the directory in which ${usageself} lives.
	HERE
	)"
	if [ -n "${usageflag}" ]
	then printf "%s\n" "${usagemessage}"; exit 0
	else printf "%s\n" "${usagemessage}" >&2; exit 1
	fi
}


#
# Print an error message, then print usage and exit.
#
errmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" >&2
	usage
}


#
# Remove a file or show an error message.
#
rmfile() {
	if [ -z "$1" ]; then return; fi
	if [ -e "$1" ]; then rm -f -- "$1" || { errmsg "Can't rm file '$1'."; }; else errmsg "File to rm '$1' doesn't exist."; fi
}


#
# Remove an empty directory or show an error message.
#
rmdirectory() {
	if [ -z "$1" ]; then return; fi

	if [ -d "$1" ]
	then rmdir -- "$1" || { errmsg "Cant rmdir directory '$1'."; }
	else errmsg "Directory to rmdir '$1' doesn't exist."
	fi
}


#
# Process command line options and initialize variables.
#
setoptions() {
	dateformat="%A, %B %e, %Y"
	if [ -r "dateformat" ]
	then
		junk="$( sed -n "1p" "dateformat" )"
		if [ "#!/bin/false" = "${junk}" ]; then . "dateformat"; fi
	fi

	unset SRC_DIR DST_DIR TMP_DIR TPL_DIR FILE XLAT XLAT_OPTS1 XLAT_OPTS2 XLAT_OPTS3

	while getopts 's:d:t:i:x:1:2:3:f:h?' c
	do
		case $c in
		s)  SRC_DIR="${OPTARG}"     ;;
		d)  DST_DIR="${OPTARG}"     ;;
		t)  TMP_DIR="${OPTARG}"     ;;
		i)  TPL_DIR="${OPTARG}"     ;;
		x)  XLAT="${OPTARG}"        ;;
		1)  XLAT_OPTS1="${OPTARG}"  ;;
		2)  XLAT_OPTS2="${OPTARG}"  ;;
		3)  XLAT_OPTS3="${OPTARG}"  ;;
		f)  FILE="${OPTARG}"        ;;
		h)  usage help              ;;
		?)  usage help              ;;
		esac
	done

	[ -z "${FILE}" ]    && { errmsg "Missing -f option: no input file."; }
	[ -z "${TMP_DIR}" ] && { errmsg "Missing -t option: no temporary directory."; }
	[ -z "${DST_DIR}" ] && { errmsg "Missing -d option: no destination directory."; }
	[ -z "${SRC_DIR}" ] && { errmsg "Missing -s option: no source directory."; }
	[ -z "${TPL_DIR}" ] && { errmsg "Missing -i option: no templates directory."; }
	[ -z "${XLAT}" ]    && { errmsg "Missing -x option: no LML translation command."; }

	[ -r "${FILE}" ]    || { errmsg "Bad -f option: input file unreadable."; }
	[ -d "${TMP_DIR}" ] || { errmsg "Bad -t option: temporary directory unusable."; }
	[ -d "${DST_DIR}" ] || { errmsg "Bad -d option: destination directory unusable."; }
	[ -d "${SRC_DIR}" ] || { errmsg "Bad -s option: source directory inaccessible."; }
	[ -d "${TPL_DIR}" ] || { errmsg "Bad -i option: template directory inaccessible."; }
	which "${XLAT}" >/dev/null 2>&1 || { errmsg "Bad -x option: LML command unusable."; }

	OFILE="$( printf "%s\n" "${FILE}" | sed "s:${SRC_DIR}/:${DST_DIR}/:;s/\.txt$//;s/$/.html/" )"

	maketempdir
}


#
# Create temporary scratch directory and initialize temporary scratch files.
#
maketempdir() {
	TMP_WORK="$( mktemp -d "${TMP_DIR}/txt2html.XXXXXX" )" || { errmsg "mktemp err: '${TMP_WORK}'"; }

	touch "${TMP_WORK}/header"  	# source header piece
	touch "${TMP_WORK}/title"   	# page title
	touch "${TMP_WORK}/author"  	# page author
	touch "${TMP_WORK}/date"    	# page date (default today if empty)
	touch "${TMP_WORK}/rickdate"	# page rickdate code
	touch "${TMP_WORK}/template"	# page template file (default page.m4 if empty)
	touch "${TMP_WORK}/body"    	# source body piece
	touch "${TMP_WORK}/source"  	# M4-preprocessed body piece
	touch "${TMP_WORK}/html"    	# LML-processed body piece, now an HTML fragment
	touch "${TMP_WORK}/final"   	# finished HTML file copied to DST_DIR
}


#
# Remove temporary scratch directory and its temporary scratch files.
#
cleantempdir() {
	rmfile "${TMP_WORK}/header"
	rmfile "${TMP_WORK}/title"
	rmfile "${TMP_WORK}/author"
	rmfile "${TMP_WORK}/date"
	rmfile "${TMP_WORK}/rickdate"
	rmfile "${TMP_WORK}/template"
	rmfile "${TMP_WORK}/body"
	rmfile "${TMP_WORK}/source"
	rmfile "${TMP_WORK}/html"
	rmfile "${TMP_WORK}/final"

	rmdirectory "${TMP_WORK}"
}


#
# Phase 1: M4 preprocessing, turning M4 macros into LML/HTML bits.
#
phase1() {
	#
	# Compose M4 preprocessing options.
	#
	set -- -P
	if [ 1 -eq "$USE_TITLE"  ]; then set -- "$@" -DTITLE="${TMP_WORK}/title"; fi
	if [ 1 -eq "$USE_AUTHOR" ]; then set -- "$@" -DAUTHOR="${TMP_WORK}/author"; fi
	if [ 1 -eq "$USE_DATE"   ]
	then
		set -- "$@" -DDATE="${TMP_WORK}/date"
		if [ -f userickdate ]; then set -- "$@" -DRICKDATE="${TMP_WORK}/rickdate"; fi
	fi
	set -- "$@" -DTEMPLATE="${TPL_DIR}/${TEMPLATE}" -I"${TPL_DIR}/" -I"${SRC_DIR}/"
	dname="${FILE}"
	while [ "." != "${dname}" ]
	do
		dname="$( dirname "${dname}" )"
		if [ "." != "${dname}" ]; then set -- "$@" -I"${dname}/"; fi
	done
	#
	# Perform the M4 preprocessing.
	#
	printf " %s" "m4 ->"
	m4 "$@" "${TMP_WORK}/body" > "${TMP_WORK}/source" || \
	{
		( cat <<- HERE
		==> M4 preprocessing error.
		If the error is quotation-related, consider adding another layer of quotes in
		the macro causing the error.
		HERE
		) >&2; exit 1
	}
}


#
# Phase 2: LML translating, giving an HTML fragment ready for an HTML template.
#
phase2() {
	#
	# Compose translation options
	#
	set --
	for option in ${XLAT_OPTS1}; do set -- "$@" "${option}"; done
	set -- "$@" "${TMP_WORK}/source"
	for option in ${XLAT_OPTS2}; do set -- "$@" "${option}"; done
	set -- "$@" "${TMP_WORK}/html"
	for option in ${XLAT_OPTS3}; do set -- "$@" "${option}"; done
	#
	# Perform the translation
	#
	printf " %s" "${XLAT} ->"
	${XLAT} "$@"
}


#
# Phase 3: M4 postprocessing, inserting the HTML fragment into the page template.
#
phase3() {
	#
	# Compose M4 postprocessing options.
	#
	set -- -P
	if [ 1 -eq "$USE_TITLE"  ]; then set -- "$@" -DTITLE="${TMP_WORK}/title"; fi
	if [ 1 -eq "$USE_AUTHOR" ]; then set -- "$@" -DAUTHOR="${TMP_WORK}/author"; fi
	if [ 1 -eq "$USE_DATE"   ]
	then
		set -- "$@" -DDATE="${TMP_WORK}/date"
		if [ -f userickdate ]; then set -- "$@" -DRICKDATE="${TMP_WORK}/rickdate"; fi
	fi
	if [ -n "$( tr -d "\n" < "${TMP_WORK}/html" )" ]; then set -- "$@" -DBODY="${TMP_WORK}/html"; fi
	set -- "$@" -I"${TPL_DIR}/" -I"${SRC_DIR}/"
	dname="${FILE}"
	while [ "." != "${dname}" ]
	do
		dname="$( dirname "${dname}" )"
		if [ "." != "${dname}" ]; then set -- "$@" -I"${SRC_DIR}/${dname}/"; fi
	done
	#
	# Perform the M4 postprocessing.
	#
	printf " %s" "m4 template ->"
	m4 "$@" "${TPL_DIR}/${TEMPLATE}" > "${TMP_WORK}/final" || \
	{
		( cat <<- HERE
		==> M4 postprocessing error.
		Did you typo the %tpl template filename?  Does the template file exist where
		you think it does?
		HERE
		) >&2; exit 1
	}
}


#
# Program proper start.
#
# Process command line options and initialize scratch space.
#
setoptions "$@"


#
# Separate source file body and header.
#
# body
printf "%s\n" "m4_changequote(\`[[',\`]]')m4_dnl" > "${TMP_WORK}/body"
printf "%s\n" "m4_changecom()m4_dnl" >> "${TMP_WORK}/body"
sed -n "1,/^$/d;p" "${FILE}" >> "${TMP_WORK}/body"
#
#header
sed -n "1,/^$/p" "${FILE}" > "${TMP_WORK}/header"

#
# Extract header percent-codes:
#
# %t   - page title         - used in the HTML head as well as for the page content title
# %a   - page author        - credited in the HTML body footer as the author
# %d   - page update date   - page dates eliminate ambiguity about timeliness or context
# %tpl - page template file - M4 template to use instead of page.m4
# %dep - page dependencies  - files this page depends on (processed in makefiles.sh, not here)
#

#
# %t - page title
#
grep -e "^%t " "${TMP_WORK}/header" | sed -n "s/^%t //;p" | tr -d "\n" > "${TMP_WORK}/title"

#
# %a - page author
#
grep -e "^%a " "${TMP_WORK}/header" | sed -n "s/^%a //;p" | tr -d "\n" > "${TMP_WORK}/author"

#
# %d - page date
#
grep -e "^%d " "${TMP_WORK}/header" | sed -n "s/^%d //;p" | tr -d "\n" > "${TMP_WORK}/date"
pagedate=$( tr -d "\n" < "${TMP_WORK}/date" )
if [ -z "${pagedate}" ]; then pagedate=$( date +"${dateformat}" ); printf "%s" "${pagedate}" > "${TMP_WORK}/date"; fi
if [ -f userickdate ]; then bin/rickdate.sh "${pagedate}" > "${TMP_WORK}/rickdate"; fi

#
# %tpl - page template file
#
grep -e "^%tpl " "${TMP_WORK}/header" | sed -n "s/^%tpl //;p" | tr -d "\n" > "${TMP_WORK}/template"

#
# Set flags for processing based on file content length (zero or non).
#
if [ -n "$( tr -d "\n" < "${TMP_WORK}/title" )" ]; then USE_TITLE=1; else USE_TITLE=0; fi
if [ -n "$( tr -d "\n" < "${TMP_WORK}/author" )" ]; then USE_AUTHOR=1; else USE_AUTHOR=0; fi
if [ -n "$( tr -d "\n" < "${TMP_WORK}/date" )" ]; then USE_DATE=1; else USE_DATE=0; fi
TEMPLATE="$( tr -d "\n" < "${TMP_WORK}/template" )"; if [ -z "${TEMPLATE}" ]; then TEMPLATE="page.m4"; fi

#
# Process the body part with M4 and the LML translator.
#
printf "%s" "==> ${FILE} ->"
phase1	# Process the M4, rendering a mixed LML/HTML page fragment.
phase2	# Process the LML, rendering a purely HTML page fragment.
phase3	# Process the page template M4, inserting the page fragment into a full, conformant HTML page.
printf " %s" "${OFILE}"
#
# Proccesing complete, stage the final file.
#
mkdir -p -- "$( dirname "${OFILE}" )"
cp -f -- "${TMP_WORK}/final" "${OFILE}"

#
# All work done, clean up scratch space.
#
cleantempdir

#printf " %s\n" "done."
printf "\n"

# End.

