#!/bin/sh

# Terminal width: 132 columns.
# function definitions end at comment "# Program proper start."

#
# Print usage and exit.
#
usage() {
	usageflag="$1"
	usageself=$( basename "$0" )
	usagemessage="$( cat <<- HERE
	Upload the staged website content to the live website server using rsync.
	With default options in 'config.mk', all new and changed files will be
	uploaded, all files missing from the staged website will be deleted from
	the live website, and all unchanged files will be left as they are.

	Usage:

	${usageself} -h|-?
	${usageself} -c command -o options -s dest_dir -d address

	-h or -?     Show this help text and exit without error.
	-c command   Rsync command script or executable to use.
	-o options   Rsync options between command name and addresses.
	-s dest_dir  Source address, directory dest_dir containing staged website.
	-d address   Destination address, location of the live website.

	When the command given by -c is invoked, the parameters given by -o, -s, and
	-d are assembled and executed as follows:

	command options dest_dir/ address

	Directory options are either absolute paths or paths relative to the current
	directory, not the directory in which ${usageself} lives.
	HERE
	)"
	if [ -n "${usageflag}" ]
	then printf "%s\n" "${usagemessage}"; exit 0
	else printf "%s\n" "${usagemessage}" >&2; exit 1
	fi
}


#
# Print an error message, then print usage and exit.
#
errmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" >&2
	printf "%s\n" "Edit the RSYNC options in 'config.mk' as needed." >&2
	exit 1
}


#
# Process command line options and initialize variables.
#
setoptions() {
	unset RSYNC OPTIONS DST_DIR ADDRESS

	while getopts 'c:o:s:d:h?' c
	do
		case $c in
		c)  RSYNC="${OPTARG}"     ;;
		o)  OPTIONS="${OPTARG}"   ;;
		s)  DST_DIR="${OPTARG}"   ;;
		d)  ADDRESS="${OPTARG}"   ;;
		h)  usage help              ;;
		?)  usage help              ;;
		esac
	done

	[ -z "${RSYNC}" ]   && { errmsg "Error: no RSYNC value set."; }
	[ -z "${DST_DIR}" ] && { errmsg "Error: no DST_DIR value set."; }
	[ -z "${ADDRESS}" ] && { errmsg "Error: no ADDRESS value set."; }

	[ -e "${DST_DIR}" ] || { errmsg "Error: DST_DIR doesn't exist."; }
	[ -d "${DST_DIR}" ] || { errmsg "Error: DST_DIR isn't an accessible directory."; }

	which "${RSYNC}" >/dev/null 2>&1 || { errmsg "Error: RSYNC command unusable."; }
}


#
# Perform the rsync operation.
#
dorsync() {
	#
	# Compose translation options
	#
	set --
	for option in ${OPTIONS}; do set -- "$@" "${option}"; done
	set -- "$@" "${DST_DIR}/"
	set -- "$@" "${ADDRESS}"
	#
	# Perform the translation
	#
	printf "%s\n" "So far, so good.  You might be prompted for a password."
	printf "%s " "${RSYNC}"; printf "%s " "$@"; printf "\n"
	if "${RSYNC}" "$@"
	then
		SYNCED=0
	else
		SYNCED=$?
	fi
}


#
# Program proper start.
#
# Process command line options.
#
setoptions "$@"

#
# Checks passing, compose and perform the rsync operation.
#
dorsync

#
# All work done.
#
if [ "0" -eq "${SYNCED}" ]
then printf "%s\n" "Done."
else printf "%s\n" "Failed.  Error code: ${SYNCED}"
fi

# End.

