#!/bin/sh

# Terminal width: 132 columns.
# Function definitions end at comment "# Program proper start."

#
# Print usage and exit.
#
usage() {
	usageflag="$1"
	usageself="$( basename "$0" )"
	usagemessage="$( cat <<- HERE
	Run python's simple Web server on 127.0.0.1:8080 so you can test that
	the site was built correctly (and hopefully catch mistakes and typos)
	before you upload the site's changes.

	Usage:

	${usageself} -h|-?
	${usageself} -d dest_dir -p python

	-h or -?     Show this help text and exit without error.
	-d dest_dir  Directory dest_dir containing the staged website to test.
	-p python    Command python naming the python binary on this system.

	Directory options are either absolute paths or relative to the current
	directory, not the directory in which ${usageself} lives.
	HERE
	)"
	if [ -n "${usageflag}" ]
	then printf "%s\n" "${usagemessage}"; exit 0
	else printf "%s\n" "${usagemessage}" >&2; exit 1
	fi
}


#
# Print an error message, then print usage and exit.
#
errmsg() {
	if [ -z "$1" ]; then return; fi
	printf "%s\n" "$1" >&2
	usage
}


#
# Process command line options and initialize variables.
#
setoptions() {
	unset DST_DIR PYTHON

	while getopts 'd:p:h?' c
	do
		case $c in
		d)  DST_DIR="${OPTARG}"  ;;
		p)  PYTHON="${OPTARG}"   ;;
		h)  usage help           ;;
		?)  usage help           ;;
		esac
	done

	[ -z "${DST_DIR}" ] && { errmsg "Missing -d option: no destination directory."; }
	[ -z "${PYTHON}"  ] && { errmsg "Missing -p option: no python executable."; }

	[ -d "${DST_DIR}" ] || { errmsg "Bad -d option: destination directory inaccessible."; }

	which "${PYTHON}" >/dev/null 2>&1 || \
	{ errmsg "$(
		cat <<- HERE
		bad -p option: \$( which "${PYTHON}" ) didn't find a command.

		If python really is installed, ask your system administrator for the
		name of the actual python interpreter binary, then edit "config.mk" and
		set the PYTHON option to the actual name.  Include the full path if it
		isn't in a directory listed in PATH.

		If you are the system administrator, verify that a python interpreter is
		installed, and verify the name of the python interpreter binary.

		On Debian and Ubuntu systems:
		    $ dpkg -L python-minimal | grep -w bin

		On Red Hat and Fedora systems:
		    $ rpm -q -l python | grep -w bin

		On FreeBSD systems:
		    $ pkg info -l python37 | grep -w bin
		  or
		    $ pkg info -l \$( pkg search python | \\
		    > grep -e "^python[[:digit::]]\\{2\\}" | \\
		    > cut -d- -f1 ) | grep -w bin

		Help text:
		HERE
		)";
	}

}


#
# Program proper start.
#
# Process command line options and initialize scratch space.
#
setoptions "$@"


#
# Determine python version and, based on that, name of module to run.
#
module="http.server"
if "${PYTHON}" --version 2>&1 | grep -e "^Python 2" >/dev/null
then module="SimpleHTTPServer"
fi

#
# Run the test Web server.
#
cat << HERE

    *********************************************
    *                                           *
    *  Press ^C to stop the testing web server  *
    *                                           *
    *********************************************

         Browse to:  http://127.0.0.1:8080/

HERE
cd "${DST_DIR}"                 || { errmsg "Can't cd into DST_DIR, '${DST_DIR}'."; }
"${PYTHON}" -m "${module}" 8080 || { printf "%s\n" "Can't run: ${PYTHON} -m ${module} 8080"; }
cd "${OLDPWD}"                  || { errmsg "Can't cd back to OLDPWD, '${OLDPWD}'."; }

#
# All work done.
#
printf "%s\n" "done."
# End.

